'''Imports some SpartyJet top-tagging analyses and defines some metadata.

Also defines the basic run template for BOOST 2011, with input tools, etc.
'''
# run in batch mode for ROOT
from sys import argv
if '-b' not in argv: argv.append('-b')
from spartyjet import *

# Load all the analyses defined in the boost module in SpartyJet
allAnalyses = {'ATLAS': boost.ATLAS, 'CMS': boost.CMS, 'HEP': boost.HEP,
               'JH': boost.JohnsHopkins, 'NSub': boost.NSubjettiness,
               'Pruned': boost.Pruned, 'Trimmed': boost.Trimmed,
               'TW': boost.ThalerWang}

# How many output variables to cut on; found through trial-and-error
# TMVA scan will be over analysis.outputVariables()[:bestN]
bestNOutputVariables = {'ATLAS':   4,
                        'CMS':     2,
                        'HEP':     3,
                        'JH':      3,
                        'NSub':    2,
                        'Pruned':  2,
                        'Trimmed': 2,
                        'TW':      3
                       }


def ParameterScanList(parameterList, Npoints):
  ''' Generate a list (of lists) of parameter points.
  
  Each element of the returned list is a dictionary of {'paramName' : value}
  pairs.  The parameters to scan over are given in parameterList, which should
  be a list of Parameter objects (defined in
  spartyjet/python/spartyjet/analyses/JetAnalysisLive.py).
  '''
  # start with one parameter point, the input list.  Note that scanList is a
  # list of dictionaries.
  scanList = [dict([(p.name, p.value) for p in parameterList])]

  # if Npoints is 1, just use default values
  if Npoints == 1:
    return [{}]

  # for each parameter that has a range, make Npoints copies of the parameter
  # dictionary, varying that parameter
  for paramToScan in parameterList:
    if paramToScan.range is None: continue
    # copy the list we generated in the previous step
    scanList_tmp = scanList
    scanList = []
    # for every element in the previous list, make Npoints copies with different values of this parameter.
    for oldParamDict in scanList_tmp:
      for paramVal in [paramToScan.range[0] + (paramToScan.range[1] - paramToScan.range[0])*(n/float(Npoints-1)) for n in range(Npoints)]:
        newParamDict = dict(oldParamDict)
        newParamDict[paramToScan.name] = paramVal
        scanList.append(newParamDict)

  return scanList



def RunAnalysis(infiles, analysis, nEvents=-1, inputTools=[]):
  '''Runs a SpartyJet analysis on a list of input files.

  Returns the name of the output ROOT file.
  ''' 

  # Create a jet builder
  builder = SJ.JetBuilder(SJ.WARNING)
  # Configure input
  input = SJ.MultiInput()
  for f in infiles:
    print "Opening: ", f
    input_i = getInputMaker(f)
    from os.path import basename as bn
    input_i.set_name(bn(f).split('.')[0])
    input.addInputMaker(input_i)
  builder.configure_input(input, False) # 2nd arg: don't save inputs in output file

  # PDG id cut -- remove muons (13) and neutrinos (12, 14, 16)
  ids = stdVector(-12, 12, 14, -14, 16, -16, 13, -13)
  builder.add_jetTool_input(SJ.JetInputPdgIdSelectorTool(ids))
  # Input eta cut
  builder.add_jetTool_input(SJ.JetEtaCentralSelectorTool(-5.0,5.0))

  # extra input tools:
  for tool in inputTools:
    builder.add_jetTool_input(tool)

  # Add analysis to JetBuilder
  builder.add_analysis(analysis, False) # 2nd arg: don't save constiuent indices
  # Configure output
  #  Come up with a unique name for each temp file
  RunAnalysis.counter += 1
  outfile = infiles[0].split('/')[-1].split('_')[0].split('.')[0] + '.' + analysis.name() + '.tmp' + '.' + str(RunAnalysis.counter) + '.root'
  builder.configure_output("Jets", outfile) 
  # Run SpartyJet
  builder.print_event_every(100)
  builder.process_events(nEvents)

  return outfile

RunAnalysis.counter = 0
