'''
Several functions for using TMVA to generate mis-tag vs. efficiency scans for
some analysis.  Can either run a given analysis, scanning over input parameters,
or just read the results of a completed analysis from a ROOT file.  The former
is implemented in EfficienciesFromRun(...), the latter in
EfficienciesFromFiles(...).  Both call EfficienciesFromTrees(...).

The function EfficienciesParamScan(...) scans over input parameters by
repeatedly calling EfficienciesFromRun(...).

The "efficiencies" defined here are the number of jets passing the tagger and
the cuts found by TMVA, divided by the number of jets at the end of the
analysis, ie, the number of jets passing the pT cut (at most two per event).
Jets failing the tagger but passing the pT cut are included in the output files
as jets with p = (0,0,0,0), so we include the tagger efficiency.
'''

from EfficiencyPoint import EfficiencyPoint
from Analyses import *

def EfficienciesFromTrees(analysis, nVars, sigTree, bgTree, sigEfficiencies):
  '''Reads the results of an analysis and optimizes cuts using TMVA.
  
  Returns a list of EfficiencyPoints corresponding to the given signal
  efficiencies.  Uses TMVA to scan over output cuts and find the optimal cuts
  and BG efficiency for each signal efficiency.  The 'parameters' member of the
  EfficiencyPoints is not set.  nVars gives the number of analysis variables to
  use from analysis.outputVariables() (takes the first nVars). nVars=-1 takes
  all, nVars=-2 takes "best" N, given by analysis.bestNOutputVariables().
  '''
  from ROOT import TMVA, TFile, TCut, gROOT, kCanDelete

  # if analysis is a class type, get its name (otherwise analysis is an instance)
  try:
    analysisName = analysis.__name__.replace('Analysis', '')
  except AttributeError:
    analysisName = analysis.name()
  
  outputFile = TFile(analysisName+".TMVA.root", 'RECREATE')
  factory = TMVA.Factory("CutFinder", outputFile, "V:!Silent:!DrawProgressBar")
    
  # Set up variables
  optString = ''
  if nVars is -1: nVars = len(analysis.outputVariables())
  if nVars is -2: nVars = bestNOutputVariables[analysisName]
  for i,(var,type) in enumerate(analysis.outputVariables()[:nVars]):
    factory.AddVariable(var, type)
    if 'mass' in var:
      optString += ':CutRangeMin[{0}]=120'.format(i)
      optString += ':CutRangeMax[{0}]=1000'.format(i)
    if 'cosThetaH' in var:
      optString += ':VarProp[{0}]=FMin'.format(i)  
    if 'tau3' in var:
      optString += ':VarProp[{0}]=FMin'.format(i)  
  print 'Adding options '+optString

  factory.AddSignalTree(sigTree, 1.0)
  factory.AddBackgroundTree(bgTree, 1.0)
  
  # do these options make sense?
  # require that the mass is non-zero (ie, that tagger, if any, was passed)
  cut = TCut(analysisName+'_mass > 0')
  factory.PrepareTrainingAndTestTree(cut, "nTrain_Signal=0:nTrain_Background=0:SplitMode=Random:NormMode=NumEvents:V")
  factory.BookMethod(TMVA.Types.kCuts, "Cuts", "V:!H:FitMethod=GA:EffSel"+optString)
  
  # train, test, evaluate
  factory.TrainAllMethods()
  factory.TestAllMethods()
  factory.EvaluateAllMethods()
  
  outputFile.Close()
  
  # Now, find best BG efficiencies and cuts for each desired signal efficiency
  bgEffs = {}
  cuts = {}
  
  # since we used a preselection cut, need to account for that to get "real" effs
  massName = analysisName+'_mass'
  preSelectionEffs = []
  for tree in [sigTree, bgTree]:
    tree.Draw(massName, '', ' goff')
    total = tree.GetHistogram().GetEntries()
    tree.Draw(massName, massName+' > 0', ' goff')
    afterCut = tree.GetHistogram().GetEntries()
    preSelectionEffs.append(afterCut / total)

  (sigPreEff, bgPreEff) = tuple(preSelectionEffs)
  print 'Preselection efficiencies:', preSelectionEffs
 
  # Read info from the XML file (doesn't seem to be an easy way to do this in code...)
  import xml.etree.ElementTree as ET
  tree = ET.parse("weights/CutFinder_Cuts.weights.xml")
  weights = tree.find("Weights")
  bins = weights.findall("Bin")
  
  effPoints = dict([(sigEff, EfficiencyPoint(sigEff)) for sigEff in sigEfficiencies])

  for b in bins:
    effPoint = EfficiencyPoint()
    effPoint.signalEfficiency = round(float(b.attrib['effS']) * sigPreEff, 2)
    effPoint.BGEfficiency = float(b.attrib['effB']) * bgPreEff

    # Optimal cuts for this point
    cutDict = {}
    cutsElem = b.find("Cuts")
    for i, (var,type) in enumerate(analysis.outputVariables()[:nVars]):
      min, max = cutsElem.attrib['cutMin_'+str(i)], cutsElem.attrib['cutMax_'+str(i)]
      cutDict[var] = (float(min), float(max))
    effPoint.cuts = cutDict
    
    sigEff = effPoint.signalEfficiency
    if (sigEff in effPoints and effPoint.BGEfficiency < effPoints[sigEff].BGEfficiency and effPoint.BGEfficiency >= 0):
      effPoints[sigEff] = effPoint
  return [effPoints[eff] for eff in sigEfficiencies]


def EfficienciesFromFiles(analysis, nVars, sigFile, bgFile, sigEfficiencies):
  '''Reads the results of an analysis and optimizes cuts using TMVA.
  
  Reads analysis output from given ROOT files, then passes the TTrees to
  BGEfficiencyFromTrees.
  '''
  from ROOT import TFile
  
  sfile = TFile(sigFile)
  bfile = TFile(bgFile)
  sigTree = sfile.Get("Jets")
  bgTree = bfile.Get("Jets")
  return EfficienciesFromTrees(analysis, nVars, sigTree, bgTree, sigEfficiencies)


def EfficienciesFromRun(analysis, nVars, sigFiles, bgFiles, sigEfficiencies, nEvents=-1):
  '''Runs an analysis and optimizes cuts using TMVA.
  
  Runs a given SpartyJet analysis on given input files, then passes the output
  files to EfficienciesFromFiles.
  '''
  from ROOT import TFile

  sfileName = RunAnalysis(sigFiles, analysis, nEvents)
  bfileName = RunAnalysis(bgFiles, analysis, nEvents)
  return EfficienciesFromFiles(analysis, nVars, sfileName, bfileName, sigEfficiencies)


def EfficienciesParamScan(analysisType, nVars, nPoints, sigFiles, BGFiles, sigEfficiencies, nEvents=-1):
  '''Find approximately optimal BG efficiencies for an analysis by scanning over its input parameters.
  
  An analysis object provides its parameters and sensible scan range; the user
  just needs to specify the analysis and the number of points to scan over.
  
  For each point in parameter space, this uses TMVA to scan over cuts on output
  variables.

  The set of optimal parameters and cuts are found for each signal efficiency
  given (sigEfficiencies should be a list), returned as a list of
  EfficiencyPoints.
  '''
  # EfficiencyPoints to hold the best BG efficiencies, and corresponding cuts,params (so far)
  bestEfficiencies = [EfficiencyPoint(sigEff) for sigEff in sigEfficiencies]

  params = analysisType.parameters()
  scanList = ParameterScanList(params, nPoints)

  for i, paramDict in enumerate(scanList):
    # update status
    from subprocess import call
    call('rm -f status*', shell=True)
    call(['touch', 'status.{0}.{1}'.format(i, len(scanList))])

    analysis = analysisType(**paramDict)
    print ""
    print "Running {0} analysis with parameters {1}".format(analysis.name(), paramDict)
    effPoints = EfficienciesFromRun(analysis, nVars, sigFiles, BGFiles, sigEfficiencies, nEvents)

    #print ""
    for sigEff, BGEff, cuts in [(p.signalEfficiency, p.BGEfficiency, p.cuts) for p in effPoints]:
      #print "  Optimal variable cuts for signal (background) efficiency {0} ({1})".format(sigEff, BGEff)
      for var,(min,max) in cuts.iteritems():
        pass
        #print '    {variable}: [{cutMin}, {cutMax}]'.format(variable=var, cutMin=min, cutMax=max)
      pass
      #print ""
    
    
    for best,new in zip(bestEfficiencies, effPoints):
      if (new.BGEfficiency < best.BGEfficiency and new.BGEfficiency > 0): # sometimes TMVA gives junk negative efficiencies when statistics are low
        best.copy(new)
        best.parameters = paramDict
    print "Best efficiencies, cuts so far:"
    for effPoint in bestEfficiencies:
      print "  {0}: {1} {2} {3}".format(effPoint.signalEfficiency, effPoint.BGEfficiency, effPoint.parameters, effPoint.cuts) 
    print ""
  
  # Mathematica-style output
  print '{'+','.join(['{'+str(effPoint.signalEfficiency)+','+str(effPoint.BGEfficiency)+'}' for effPoint in bestEfficiencies])+'}'
  
  return bestEfficiencies

