#!/usr/bin/env python

# This script implements all of the top taggers for the Boost 2011
# proceedings.  Where available, parameters are for the 50% efficiency
# working point from last year.  For the most part cuts are not made, since
# this can be done afterward if the variables are stored.


# NOTE: SPARTYJETDIR/python must be in your PYTHONPATH.  You can set this by
# doing 'source setup.sh' in SPARTYJETDIR.
from sys import argv
# put ROOT in batch mode
if '-b' not in argv: argv.append('-b')
from spartyjet import *
#===============================================

# silence useless ROOT warnings
import ROOT
ROOT.gErrorIgnoreLevel = 2001

from glob import glob
infiles = glob(argv[1])
outfile = argv[2]

# --------------------- Setup/Input ----------------------------------

# Create a jet builder
builder = SJ.JetBuilder(SJ.WARNING)
# Configure inputi
input = SJ.MultiInput()
for f in infiles:
  print "Opening: ", f
  input_i = getInputMaker(f)
  from os.path import basename as bn
  input_i.set_name(bn(f).split('.')[0])
  input.addInputMaker(input_i)
builder.configure_input(input)

# PDG id cut -- remove muons (13) and neutrinos (12, 14, 16)
ids = stdVector(-12, 12, 14, -14, 16, -16, 13, -13)
builder.add_jetTool_input(SJ.JetInputPdgIdSelectorTool(ids))
# Input eta cut
builder.add_jetTool_input(SJ.JetEtaCentralSelectorTool(-5.0,5.0))

# Peter Loch's detector simulator:
#builder.add_jetTool_input(SJ.RadialSmearingTool())

# ------------------------- Analyses ---------------------------------
analyses = []

# They all start with Anti-kt 1.0 jets, so set this up and fork off
antiktFinder = SJ.FastJet.FastJetFinder('AntiKt', fj.antikt_algorithm, 1.0)
AntiKt = SJ.JetAnalysis(antiktFinder)
# cut on pT *before* forking to save time
AntiKt.add_tool(SJ.JetPtSelectorTool(200.0, 2)) # two highest-pt jets over 200
AKTparent = SJ.ForkToolParent('AntiKtParent')
AntiKt.add_tool(AKTparent)
analyses.append(AntiKt)

# recluster with CA, then fork again for different taggers
CA = SJ.JetAnalysis(SJ.ForkToolChild(AKTparent, 'CA'))
CA.add_tool(SJ.FastJet.FastJetRecluster('CAcluster', fj.cambridge_algorithm, 100.0))
CAparent = SJ.ForkToolParent('CAParent')
CA.add_tool(CAparent)
analyses.append(CA)

# recluster with kT
Kt = SJ.JetAnalysis(SJ.ForkToolChild(AKTparent, 'KT'))
Kt.add_tool(SJ.FastJet.FastJetRecluster('KTcluster', fj.kt_algorithm, 100.0))
KTparent = SJ.ForkToolParent('KTParent')
Kt.add_tool(KTparent)
analyses.append(Kt)

# do some forks before measurement, just for clarity of output
ATLAS = SJ.JetAnalysis(SJ.ForkToolChild(KTparent, 'ATLAS'))
analyses.append(ATLAS)
TW = SJ.JetAnalysis(SJ.ForkToolChild(KTparent, 'TW'))
analyses.append(TW)
NSub = SJ.JetAnalysis(SJ.ForkToolChild(AKTparent, 'NSub'))
analyses.append(NSub)

# pruning
big_CA_def = fj.JetDefinition(fj.cambridge_algorithm, 100.0)
Pruned50 = SJ.JetAnalysis(SJ.ForkToolChild(AKTparent, "Pruned"))
pruner50 = fj.Pruner(fj.cambridge_algorithm, 0.05, 0.1)
Pruned50.add_tool(SJ.FastJet.TransformerTool("Pruner", pruner50))
analyses.append(Pruned50)

# trimming
Trimmed = SJ.JetAnalysis(SJ.ForkToolChild(CAparent, "Trimmed"))
trimmer = fj.Filter(0.35, fj.SelectorPtFractionMin(0.03))
Trimmed.add_tool(SJ.FastJet.TransformerTool("TrimmerTool", trimmer))
analyses.append(Trimmed)

# filtering
Filtered = SJ.JetAnalysis(SJ.ForkToolChild(CAparent, "Filtered"))
filter = fj.Filter(0.35, fj.SelectorNHardest(3))
Filtered.add_tool(SJ.FastJet.TransformerTool("FilterTool", filter))
analyses.append(Filtered)

# JH top tagger
JH50 = SJ.JetAnalysis(SJ.ForkToolChild(CAparent, "JH"))
JH50transformer = fj.JHTopTagger(0.04, 0.19, 1.0, 81.0)
JH50.add_tool(SJ.FastJet.TransformerTool("JHTagger", JH50transformer))
analyses.append(JH50)

# CMS top tagger
CMS = SJ.JetAnalysis(SJ.ForkToolChild(CAparent, "CMS"))
# measure mass *before* "pruning" step
massFunction = SJ.FastJet.MassFunction()
CMS.add_tool(SJ.FastJet.PseudoJetMomentTool(double)(massFunction, "AKTmass"))
CMStransformer = fj.CMSTopTagger(0.05, 0.4, 0.0004)
CMS.add_tool(SJ.FastJet.TransformerTool("CMSTagger", CMStransformer))
analyses.append(CMS)

# HEP top tagger
HEP = SJ.JetAnalysis(SJ.ForkToolChild(CAparent, "HEP"))
HEPtransformer = fj.HEPTopTagger()
HEP.add_tool(SJ.FastJet.TransformerTool("HEPTagger", HEPtransformer))
analyses.append(HEP)

[builder.add_analysis(a) for a in analyses]


# ------------------ Measurements ------------------------------------

# ATLAS z_cut variables
indices = [1, 2, 3, 4]
zCutFunctions = [fj.zCutFunction(i) for i in indices]
[ATLAS.add_tool(SJ.FastJet.PseudoJetMomentTool(double)(zCut, "zcut"+str(i))) for (zCut,i) in zip(zCutFunctions, indices)]

# Thaler-Wang z_cell
zCellFunction = fj.zCellFunction()
TW.add_tool(SJ.FastJet.PseudoJetMomentTool(double)(zCellFunction, "zcell"))

# N-Subjettiness
nsub_tools = []
for i in [1,2,3]:
   nsub_tools += [fj.Nsubjettiness(i, Njettiness.kt_axes, 1.0, 1.0)]
   NSub.add_tool(SJ.FastJet.PseudoJetMomentTool(double)(nsub_tools[-1], "tau"+str(i)+"_kt"))
   # these ones are the ones actually used in the report
   nsub_tools += [fj.Nsubjettiness(i, Njettiness.onepass_kt_axes, 1.0, 1.0)]
   NSub.add_tool(SJ.FastJet.PseudoJetMomentTool(double)(nsub_tools[-1], "tau"+str(i)))

# Add a min-mass tool to analyses that don't have a different W identifier
minMass = fj.MinMassFunction()
for a in [Pruned50, Filtered, Trimmed, ATLAS, TW, NSub]:
  a.add_tool(SJ.FastJet.PseudoJetMomentTool(double)(minMass, "mW"))

# --------------- Setup output and RUN -------------------------------------

# Configure output
builder.configure_output("Jets", outfile)

# Run SpartyJet
builder.print_event_every(100)
builder.process_events()

# Save this script in the ROOT file (needs to go after process_events or it
#  gets over-written!)
writeCurrentFile(outfile)

